'use strict';

/**
* Copyright (c) 2017 Copyright taijin All Rights Reserved.
* Author: lipengxiang
* Date: 2018-07-07 14:24
* Desc: 
*/

var assert = require('assert');
var routerScrollBehavior = require('./routerScrollBehavior');

module.exports = setupConfig;

/**
* @desc: 设置nuxt配置文件.
* @param cfg: {
              }
*/
function setupConfig(nuxtConfig, cfg) {

  cfg = cfg || {};
  cfg.history_hash = nuxtConfig.mode=='history'? false: true;

  setupMeta(nuxtConfig);
  setupRouter(nuxtConfig, cfg);
  setupCss(nuxtConfig);
  setupBuild(nuxtConfig, cfg);

  return nuxtConfig;
}

/**
* @desc: 初始化meta配置.
*/
function setupMeta(nuxtConfig) {
  if (!nuxtConfig.head) {
    nuxtConfig.head = {};
  }
  let head = nuxtConfig.head;
  if (!head.meta) head.meta = [];
  let meta = head.meta;

  // value2在key2中存在则满足.
  function addMeta(key, value, key2, value2, add_object, add_comment) {
    assert(key && add_object);
    let i = 0
    let content;
    for (i = 0; i < meta.length; i++) {
      const element = meta[i];
      if (value) {
        if (element[key] == value) {
          if (key2 && value2) {
            if (key2 == 'content' && element[key2]) {
              content = element;
            }
            if (element[key2] && element[key2].indexOf(value2) >= 0)
              break;
          } else {
            break;
          }
        }
      } else {
        if (element[key]) {
          if (key2 && value2) {
            if (key2 == 'content' && element[key2]) {
              content = element;
            }
            if (element[key2] && element[key2].indexOf(value2) >= 0)
              break;
          } else {
            break;
          }
        }
      }
    }
    if (i >= meta.length) {
      if (add_comment) console.warn(add_comment);
      if (content) {
        assert(value == 'viewport' && add_object.content);
        content['content'] = content['content'] + ', ' + add_object.content;
      }
      else {
        meta.push(add_object);
      }
    }
  }

  /**
   * @desc common.
   */
  addMeta('name', 'viewport', 'content', 'width=', {
    name: 'viewport',
    content: 'width=device-width'
  }, 'Will add <meta name="viewport" content="width=device-width"> ');

  addMeta('name', 'viewport', 'content', 'initial-scale=', {
    name: 'viewport',
    content: 'initial-scale=1.0'
  }, 'Will add <meta name="viewport" content="initial-scale=1.0"> ');

  addMeta('name', 'viewport', 'content', 'minimum-scale=', {
    name: 'viewport',
    content: 'minimum-scale=1.0'
  }, 'Will add <meta name="viewport" content="minimum-scale=1.0"> ');

  addMeta('name', 'viewport', 'content', 'maximum-scale=', {
    name: 'viewport',
    content: 'maximum-scale=1.0'
  }, 'Will add <meta name="viewport" content="maximum-scale=1.0"> ');

  addMeta('name', 'viewport', 'content', 'user-scalable=', {
    name: 'viewport',
    content: 'user-scalable=no'
  }, 'Will add <meta name="viewport" content="user-scalable=no"> ');

  /**
   * @desc ie.
   */

  // 'http-equiv': 'X-UA-Compatible'.
  addMeta('http-equiv', null, null, null, {
    'http-equiv': 'X-UA-Compatible',
    content: 'IE=edge,chrome=1'
  }, 'Will add <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">');

  // 'name': 'HandheldFriendly'.
  addMeta('name', 'HandheldFriendly', null, null, {
    name: 'HandheldFriendly',
    content: 'true'
  }, 'Will add <meta name="HandheldFriendly" content="true">');

  /**
   * @desc apple.
   */

  // // 'name': 'viewport', content=minimal-ui.
  // addMeta('name', 'viewport', 'content', 'minimal-ui', {
  //   name: 'viewport',
  //   content: 'minimal-ui'
  // }, 'Will add <meta name="viewport" content="minimal-ui"> ');

  // iphoneX 充满.
  addMeta('name', 'viewport', 'content', 'viewport-fit=', {
    name: 'viewport',
    content: 'viewport-fit=cover'
  }, 'Will add <meta name="viewport" content="viewport-fit=cover"> ');

  // 'name': 'apple-touch-fullscreen'.
  addMeta('name', 'apple-touch-fullscreen', null, null, {
    name: 'apple-touch-fullscreen',
    content: 'yes'
  }, 'Will add <meta name="apple-touch-fullscreen" content="yes">');

  // 'name': 'apple-mobile-web-app-capable'.
  addMeta('name', 'apple-mobile-web-app-capable', null, null, {
    name: 'apple-mobile-web-app-capable',
    content: 'yes'
  }, 'Will add <meta name="apple-mobile-web-app-capable" content="yes">');

  // 'name': 'apple-mobile-web-app-status-bar-style'.
  addMeta('name', 'apple-mobile-web-app-status-bar-style', null, null, {
    name: 'apple-mobile-web-app-status-bar-style',
    content: 'black-translucent'
  }, 'Will add <meta name="apple-mobile-web-app-status-bar-style" content="black-translucent">');

  /**
   * @desc uc
   */

  addMeta('name', 'full-screen', null, null, {
    name: 'full-screen',
    content: 'yes'
  }, 'Will add <meta name="full-screen" content="yes">');

  addMeta('name', 'browsermode', null, null, {
    name: 'browsermode',
    content: 'application'
  }, 'Will add <meta name="browsermode" content="application">');

  /**
   * @desc qq
   */

  addMeta('name', 'x5-fullscreen', null, null, {
    name: 'x5-fullscreen',
    content: 'true'
  }, 'Will add <meta name="x5-fullscreen" content="true">');

  addMeta('name', 'x5-page-mode', null, null, {
    name: 'x5-page-mode',
    content: 'app'
  }, 'Will add <meta name="x5-page-mode" content="app">');

  /**
   * @desc 360
   */

  addMeta('name', 'renderer', null, null, {
    name: 'renderer',
    content: 'webkit'
  }, 'Will add <meta name="renderer" content="webkit">');
}

/**
* @desc: 初始化router配置.
*/
function setupRouter(nuxtConfig, cfg) {
  if (!nuxtConfig.router) {
    nuxtConfig.router = {};
  }
  let router = nuxtConfig.router;

  // mode.
  if (cfg.history_hash) {
    if (router.mode && router.mode != 'hash') {
      console.warn('Will set router mode: hash');
    }
    // nuxtConfig.mode = 'spa';
    router.mode = 'hash';
  }
  else {
    if (router.mode && router.mode != 'history') {
      console.warn('Will set router mode: history');
    }
    router.mode = 'history';
  }

  // fallback.
  if (router.hasOwnProperty('fallback') && router.fallback) {
    console.warn('Will set router fallback: false');
  }
  router.fallback = false;

  // linkActiveClass.
  if (!router.hasOwnProperty('linkActiveClass')) {
    router.linkActiveClass = 'febsvue-active-link';
  }

  if (router.scrollBehavior) {
    console.warn('Will reset router.scrollBehavior');
  }
  router.scrollBehavior = routerScrollBehavior;
}

/**
* @desc: 初始化css配置.
*/
function setupCss(nuxtConfig) {
  if (!nuxtConfig.css) {
    nuxtConfig.css = [];
  }
  let css = nuxtConfig.css;

  // if (css.indexOf('febs-ui/febsui.css') < 0) {
  //   css.push('febs-ui/febsui.css');
  // }
}

/**
 * @desc: 初始化build配置.
 */
function setupBuild(nuxtConfig, cfg) {
  if (!nuxtConfig.build) {
    nuxtConfig.build = {};
  }
  let build = nuxtConfig.build;

  // babel.
  // if (!build.babel) {
  //   build.babel = {
  //     presets: ['es2015', 'stage-0', 'es2015-loose'],
  //     plugins: [
  //       'transform-runtime',
  //       // 'transform-es3-property-literals',
  //       // 'transform-es3-member-expression-literals',
  //       // 'transform-es2015-modules-simple-commonjs',
  //     ]
  //   };
  // }

  // filenames.
  if (!build.filenames) {
    if (nuxtConfig.dev) {
      build.filenames = {
        css: '[name].css',
        manifest: 'manifest.js',
        vendor: 'vendor.js',
        app: 'app.js',
        chunk: '[name].js'
      };
    }
    else {
      build.filenames = {
        css: '[name].[contenthash].css',
        manifest: 'manifest.[hash].js',
        vendor: 'vendor.[chunkhash].js',
        app: 'app.[chunkhash].js',
        chunk: '[name].[chunkhash].js'
      };
    }
  }
  else {
    if (nuxtConfig.dev) {
      for (const key in build.filenames) {
        let i = build.filenames[key].indexOf('.[contenthash]');
        if (i >= 0) {
          build.filenames[key] = build.filenames[key].substr(0, i) + build.filenames[key].substring(i+'.[contenthash]'.length, -1);
        }
        i = build.filenames[key].indexOf('.[chunkhash]');
        if (i >= 0) {
          build.filenames[key] = build.filenames[key].substr(0, i) + build.filenames[key].substring(i+'.[chunkhash]'.length, -1);
        }
      }
    }
  }

  // vendor.
  if (!build.vendor) {
    build.vendor = [];
  }

  if (build.vendor.indexOf('febs') < 0) {
    build.vendor.splice(0, 0, 'febs');
  }

  if (build.vendor.indexOf('febs-ui') < 0) {
    build.vendor.push('febs-ui');
  }
  
  if (build.vendor.indexOf('febs-vue') < 0) {
    build.vendor.push('febs-vue');
  }

  // // splitChunks
  // if (!build.splitChunks) {
  //   build.splitChunks = {};
  // }

  // if (build.vendor.indexOf('febs') < 0) {
  //   build.vendor.splice(0, 0, 'febs');
  // }

  // if (build.vendor.indexOf('febs-ui') < 0) {
  //   build.vendor.push('febs-ui');
  // }
  
  // if (build.vendor.indexOf('febs-vue') < 0) {
  //   build.vendor.push('febs-vue');
  // }
  
  // extend.
  let extend = build.extend;
  build.extend = function(config, cfg) {
                                    // if (cfg.isDev && cfg.isClient) {
                                    //   config.module.rules.push({
                                    //     enforce: 'pre',
                                    //     test: /\.(js|vue)$/,
                                    //     loader: 'eslint-loader',
                                    //     exclude: /(node_modules)/
                                    //   });
                                    // }

                                    if (cfg.isDev && cfg.isServer) {
                                    }

                                    // client.
                                    if (cfg.isClient) {
                                      config.resolve.alias['febs'] = 'febs-browser';
                                    }

                                    if (cfg.isDev) {
                                      config.devtool = 'cheap-module-eval-source-map';
                                    } else {
                                      config.devtool = false; // 'cheap-module-source-map';
                                      config.mode = 'production';
                                      // config.devtool = 'hidden-source-map';
                                    }

                                    // old extend.
                                    if (extend) {
                                      extend(config, cfg);
                                    }
                                  }

}